@@ -109,9 +109,12 @@ find_win_for_curbuf(void)
 {
     wininfo_T *wip;
 
+    // The b_wininfo list should have the windows that recently contained the
+    // buffer, going over this is faster than going over all the windows.
+    // Do check the buffer is still there.
     FOR_ALL_BUF_WININFO(curbuf, wip)
     {
-	if (wip->wi_win != NULL)
+	if (wip->wi_win != NULL && wip->wi_win->w_buffer == curbuf)
 	{
 	    curwin = wip->wi_win;
 	    break;
@@ -119,6 +122,62 @@ find_win_for_curbuf(void)
     }
 }
 
+typedef struct {
+    win_T	*cob_curwin_save;
+    aco_save_T	cob_aco;
+    int		cob_using_aco;
+    int		cob_save_VIsual_active;
+} cob_T;
+
+/*
+ * Used before making a change in "buf", which is not the current one: Make
+ * "buf" the current buffer and find a window for this buffer, so that side
+ * effects are done correctly (e.g., adjusting marks).
+ *
+ * Information is saved in "cob" and MUST be restored by calling
+ * change_other_buffer_restore().
+ *
+ * If this fails then "curbuf" will not be equal to "buf".
+ */
+    static void
+change_other_buffer_prepare(cob_T *cob, buf_T *buf)
+{
+    CLEAR_POINTER(cob);
+
+    // Set "curbuf" to the buffer being changed.  Then make sure there is a
+    // window for it to handle any side effects.
+    cob->cob_save_VIsual_active = VIsual_active;
+    VIsual_active = FALSE;
+    cob->cob_curwin_save = curwin;
+    curbuf = buf;
+    find_win_for_curbuf();  // simplest: find existing window for "buf"
+
+    if (curwin->w_buffer != buf)
+    {
+	// No existing window for this buffer.  It is dangerous to have
+	// curwin->w_buffer differ from "curbuf", use the autocmd window.
+	curbuf = curwin->w_buffer;
+	aucmd_prepbuf(&cob->cob_aco, buf);
+	if (curbuf == buf)
+	    cob->cob_using_aco = TRUE;
+    }
+}
+
+    static void
+change_other_buffer_restore(cob_T *cob)
+{
+    if (cob->cob_using_aco)
+    {
+	aucmd_restbuf(&cob->cob_aco);
+    }
+    else
+    {
+	curwin = cob->cob_curwin_save;
+	curbuf = curwin->w_buffer;
+    }
+    VIsual_active = cob->cob_save_VIsual_active;
+}
+
 /*
  * Set line or list of lines in buffer "buf" to "lines".
  * Any type is allowed and converted to a string.
@@ -137,14 +196,11 @@ set_buffer_lines(
     listitem_T	*li = NULL;
     long	added = 0;
     linenr_T	append_lnum;
-    buf_T	*curbuf_save = NULL;
-    win_T	*curwin_save = NULL;
-    int		is_curbuf = buf == curbuf;
-    int		save_VIsual_active = VIsual_active;
 
     // When using the current buffer ml_mfp will be set if needed.  Useful when
     // setline() is used on startup.  For other buffers the buffer must be
     // loaded.
+    int is_curbuf = buf == curbuf;
     if (buf == NULL || (!is_curbuf && buf->b_ml.ml_mfp == NULL) || lnum < 1)
     {
 	rettv->vval.v_number = 1;	// FAIL
@@ -153,14 +209,11 @@ set_buffer_lines(
 	return;
     }
 
+    // After this don't use "return", goto "cleanup"!
+    cob_T cob;
     if (!is_curbuf)
-    {
-	VIsual_active = FALSE;
-	curbuf_save = curbuf;
-	curwin_save = curwin;
-	curbuf = buf;
-	find_win_for_curbuf();
-    }
+	// set "curbuf" to "buf" and find a window for this buffer
+	change_other_buffer_prepare(&cob, buf);
 
     if (append)
 	// appendbufline() uses the line number below which we insert
@@ -175,9 +228,7 @@ set_buffer_lines(
 	l = lines->vval.v_list;
 	if (l == NULL || list_len(l) == 0)
 	{
-	    // set proper return code
-	    if (lnum > curbuf->b_ml.ml_line_count)
-		rettv->vval.v_number = 1;	// FAIL
+	    // not appending anything always succeeds
 	    goto done;
 	}
 	CHECK_LIST_MATERIALIZE(l);
@@ -263,11 +314,7 @@ set_buffer_lines(
 
 done:
     if (!is_curbuf)
-    {
-	curbuf = curbuf_save;
-	curwin = curwin_save;
-	VIsual_active = save_VIsual_active;
-    }
+	change_other_buffer_restore(&cob);
 }
 
 /*
@@ -505,12 +552,9 @@ f_deletebufline(typval_T *argvars, typval_T *rettv)
     linenr_T	lnum;
     long	count;
     int		is_curbuf;
-    buf_T	*curbuf_save = NULL;
-    win_T	*curwin_save = NULL;
     tabpage_T	*tp;
     win_T	*wp;
     int		did_emsg_before = did_emsg;
-    int		save_VIsual_active = VIsual_active;
 
     rettv->vval.v_number = 1;	// FAIL by default
 
@@ -523,7 +567,6 @@ f_deletebufline(typval_T *argvars, typval_T *rettv)
     buf = tv_get_buf(&argvars[0], FALSE);
     if (buf == NULL)
 	return;
-    is_curbuf = buf == curbuf;
 
     first = tv_get_lnum_buf(&argvars[1], buf);
     if (did_emsg > did_emsg_before)
@@ -537,14 +580,13 @@ f_deletebufline(typval_T *argvars, typval_T *rettv)
 			   || first > buf->b_ml.ml_line_count || last < first)
 	return;
 
+    // After this don't use "return", goto "cleanup"!
+    is_curbuf = buf == curbuf;
+    cob_T cob;
     if (!is_curbuf)
-    {
-	VIsual_active = FALSE;
-	curbuf_save = curbuf;
-	curwin_save = curwin;
-	curbuf = buf;
-	find_win_for_curbuf();
-    }
+	// set "curbuf" to "buf" and find a window for this buffer
+	change_other_buffer_prepare(&cob, buf);
+
     if (last > curbuf->b_ml.ml_line_count)
 	last = curbuf->b_ml.ml_line_count;
     count = last - first + 1;
@@ -558,38 +600,31 @@ f_deletebufline(typval_T *argvars, typval_T *rettv)
     }
 
     if (u_save(first - 1, last + 1) == FAIL)
-    {
-	rettv->vval.v_number = 1;	// FAIL
-    }
-    else
-    {
-	for (lnum = first; lnum <= last; ++lnum)
-	    ml_delete_flags(first, ML_DEL_MESSAGE);
+	goto cleanup;
 
-	FOR_ALL_TAB_WINDOWS(tp, wp)
-	    if (wp->w_buffer == buf)
-	    {
-		if (wp->w_cursor.lnum > last)
-		    wp->w_cursor.lnum -= count;
-		else if (wp->w_cursor.lnum > first)
-		    wp->w_cursor.lnum = first;
-		if (wp->w_cursor.lnum > wp->w_buffer->b_ml.ml_line_count)
-		    wp->w_cursor.lnum = wp->w_buffer->b_ml.ml_line_count;
-		wp->w_valid = 0;
-		if (wp->w_cursor.lnum <= wp->w_topline)
-		    wp->w_topline = 1;
-	    }
-	check_cursor_col();
-	deleted_lines_mark(first, count);
-    }
+    for (lnum = first; lnum <= last; ++lnum)
+	ml_delete_flags(first, ML_DEL_MESSAGE);
 
-    if (!is_curbuf)
-    {
-	curbuf = curbuf_save;
-	curwin = curwin_save;
-	VIsual_active = save_VIsual_active;
-    }
+    FOR_ALL_TAB_WINDOWS(tp, wp)
+	if (wp->w_buffer == buf)
+	{
+	    if (wp->w_cursor.lnum > last)
+		wp->w_cursor.lnum -= count;
+	    else if (wp->w_cursor.lnum > first)
+		wp->w_cursor.lnum = first;
+	    if (wp->w_cursor.lnum > wp->w_buffer->b_ml.ml_line_count)
+		wp->w_cursor.lnum = wp->w_buffer->b_ml.ml_line_count;
+	    wp->w_valid = 0;
+	    if (wp->w_cursor.lnum <= wp->w_topline)
+		wp->w_topline = 1;
+	}
+    check_cursor_col();
+    deleted_lines_mark(first, count);
     rettv->vval.v_number = 0; // OK
+
+cleanup:
+    if (!is_curbuf)
+	change_other_buffer_restore(&cob);
 }
 
 /*
@@ -618,6 +653,7 @@ get_buffer_info(buf_T *buf)
     dict_add_number(dict, "changedtick", CHANGEDTICK(buf));
     dict_add_number(dict, "hidden",
 			    buf->b_ml.ml_mfp != NULL && buf->b_nwindows == 0);
+    dict_add_number(dict, "command", buf == cmdwin_buf);
 
     // Get a reference to buffer variables
     dict_add_dict(dict, "variables", buf->b_vars);
@@ -785,10 +821,11 @@ get_buffer_lines(
 }
 
 /*
- * "getbufline()" function
+ * "retlist" TRUE: "getbufline()" function
+ * "retlist" FALSE: "getbufoneline()" function
  */
-    void
-f_getbufline(typval_T *argvars, typval_T *rettv)
+    static void
+getbufline(typval_T *argvars, typval_T *rettv, int retlist)
 {
     linenr_T	lnum = 1;
     linenr_T	end = 1;
@@ -813,7 +850,25 @@ f_getbufline(typval_T *argvars, typval_T *rettv)
 	    end = tv_get_lnum_buf(&argvars[2], buf);
     }
 
-    get_buffer_lines(buf, lnum, end, TRUE, rettv);
+    get_buffer_lines(buf, lnum, end, retlist, rettv);
+}
+
+/*
+ * "getbufline()" function
+ */
+    void
+f_getbufline(typval_T *argvars, typval_T *rettv)
+{
+    getbufline(argvars, rettv, TRUE);
+}
+
+/*
+ * "getbufoneline()" function
+ */
+    void
+f_getbufoneline(typval_T *argvars, typval_T *rettv)
+{
+    getbufline(argvars, rettv, FALSE);
 }
 
 /*
