@@ -151,7 +151,7 @@ vim_strsave_shellescape(char_u *string, int do_special, int do_newline)
     char_u	*p;
     char_u	*d;
     char_u	*escaped_string;
-    int		l;
+    size_t	l;
     int		csh_like;
     int		fish_like;
     char_u	*shname;
@@ -272,8 +272,9 @@ vim_strsave_shellescape(char_u *string, int do_special, int do_newline)
 	    if (do_special && find_cmdline_var(p, &l) >= 0)
 	    {
 		*d++ = '\\';		// insert backslash
-		while (--l >= 0)	// copy the var
-		    *d++ = *p++;
+		memcpy(d, p, l);	// copy the var
+		d += l;
+		p += l;
 		continue;
 	    }
 	    if (*p == '\\' && fish_like)
@@ -588,6 +589,33 @@ vim_strnicmp(char *s1, char *s2, size_t len)
 }
 #endif
 
+/*
+ * Compare two ASCII strings, for length "len", ignoring case, ignoring locale
+ * (mostly matters for turkish locale where i I might be different).
+ * return 0 for match, < 0 for smaller, > 0 for bigger
+ */
+    int
+vim_strnicmp_asc(char *s1, char *s2, size_t len)
+{
+    int                i;
+    int                save_cmp_flags = cmp_flags;
+
+    cmp_flags |= CMP_KEEPASCII;		// compare by ASCII value, ignoring locale
+    while (len > 0)
+    {
+       i = vim_tolower(*s1) - vim_tolower(*s2);
+       if (i != 0)
+           break;			// this character is different
+       if (*s1 == NUL)
+           break;			// strings match until NUL
+       ++s1;
+       ++s2;
+       --len;
+    }
+    cmp_flags = save_cmp_flags;
+    return i;
+}
+
 /*
  * Search for first occurrence of "c" in "string".
  * Version of strchr() that handles unsigned char strings with characters from
@@ -1036,7 +1064,7 @@ string_reduce(
  * Implementation of "byteidx()" and "byteidxcomp()" functions
  */
     static void
-byteidx_common(typval_T *argvars, typval_T *rettv, int comp UNUSED)
+byteidx_common(typval_T *argvars, typval_T *rettv, int comp)
 {
     rettv->vval.v_number = -1;
 
@@ -2496,7 +2524,8 @@ format_overflow_error(const char *pstart)
 get_unsigned_int(
     const char *pstart,
     const char **p,
-    unsigned int *uj)
+    unsigned int *uj,
+    int overflow_err)
 {
     *uj = **p - '0';
     ++*p;
@@ -2509,8 +2538,13 @@ get_unsigned_int(
 
     if (*uj > MAX_ALLOWED_STRING_WIDTH)
     {
-	format_overflow_error(pstart);
-	return FAIL;
+	if (overflow_err)
+	{
+	    format_overflow_error(pstart);
+	    return FAIL;
+	}
+	else
+	    *uj = MAX_ALLOWED_STRING_WIDTH;
     }
 
     return OK;
@@ -2583,7 +2617,7 @@ parse_fmt_types(
 		// Positional argument
 		unsigned int uj;
 
-		if (get_unsigned_int(pstart, &p, &uj) == FAIL)
+		if (get_unsigned_int(pstart, &p, &uj, tvs != NULL) == FAIL)
 		    goto error;
 
 		pos_arg = uj;
@@ -2624,7 +2658,7 @@ parse_fmt_types(
 		    // Positional argument field width
 		    unsigned int uj;
 
-		    if (get_unsigned_int(arg + 1, &p, &uj) == FAIL)
+		    if (get_unsigned_int(arg + 1, &p, &uj, tvs != NULL) == FAIL)
 			goto error;
 
 		    if (*p != '$')
@@ -2655,7 +2689,7 @@ parse_fmt_types(
 		const char *digstart = p;
 		unsigned int uj;
 
-		if (get_unsigned_int(digstart, &p, &uj) == FAIL)
+		if (get_unsigned_int(digstart, &p, &uj, tvs != NULL) == FAIL)
 		    goto error;
 
 		if (*p == '$')
@@ -2679,7 +2713,7 @@ parse_fmt_types(
 			// Parse precision
 			unsigned int uj;
 
-			if (get_unsigned_int(arg + 1, &p, &uj) == FAIL)
+			if (get_unsigned_int(arg + 1, &p, &uj, tvs != NULL) == FAIL)
 			    goto error;
 
 			if (*p == '$')
@@ -2711,7 +2745,7 @@ parse_fmt_types(
 		    const char *digstart = p;
 		    unsigned int uj;
 
-		    if (get_unsigned_int(digstart, &p, &uj) == FAIL)
+		    if (get_unsigned_int(digstart, &p, &uj, tvs != NULL) == FAIL)
 			goto error;
 
 		    if (*p == '$')
@@ -3024,7 +3058,7 @@ vim_vsnprintf_typval(
 		const char *digstart = p;
 		unsigned int uj;
 
-		if (get_unsigned_int(digstart, &p, &uj) == FAIL)
+		if (get_unsigned_int(digstart, &p, &uj, tvs != NULL) == FAIL)
 		    goto error;
 
 		pos_arg = uj;
@@ -3066,7 +3100,7 @@ vim_vsnprintf_typval(
 		    // Positional argument field width
 		    unsigned int uj;
 
-		    if (get_unsigned_int(digstart, &p, &uj) == FAIL)
+		    if (get_unsigned_int(digstart, &p, &uj, tvs != NULL) == FAIL)
 			goto error;
 
 		    arg_idx = uj;
@@ -3084,8 +3118,13 @@ vim_vsnprintf_typval(
 
 		if (j > MAX_ALLOWED_STRING_WIDTH)
 		{
-		    format_overflow_error(digstart);
-		    goto error;
+		    if (tvs != NULL)
+		    {
+			format_overflow_error(digstart);
+			goto error;
+		    }
+		    else
+			j = MAX_ALLOWED_STRING_WIDTH;
 		}
 
 		if (j >= 0)
@@ -3103,15 +3142,9 @@ vim_vsnprintf_typval(
 		const char *digstart = p;
 		unsigned int uj;
 
-		if (get_unsigned_int(digstart, &p, &uj) == FAIL)
+		if (get_unsigned_int(digstart, &p, &uj, tvs != NULL) == FAIL)
 		    goto error;
 
-		if (uj > MAX_ALLOWED_STRING_WIDTH)
-		{
-		    format_overflow_error(digstart);
-		    goto error;
-		}
-
 		min_field_width = uj;
 	    }
 
@@ -3128,15 +3161,9 @@ vim_vsnprintf_typval(
 		    const char *digstart = p;
 		    unsigned int uj;
 
-		    if (get_unsigned_int(digstart, &p, &uj) == FAIL)
+		    if (get_unsigned_int(digstart, &p, &uj, tvs != NULL) == FAIL)
 			goto error;
 
-		    if (uj > MAX_ALLOWED_STRING_WIDTH)
-		    {
-			format_overflow_error(digstart);
-			goto error;
-		    }
-
 		    precision = uj;
 		}
 		else if (*p == '*')
@@ -3151,7 +3178,7 @@ vim_vsnprintf_typval(
 			// positional argument
 			unsigned int uj;
 
-			if (get_unsigned_int(digstart, &p, &uj) == FAIL)
+			if (get_unsigned_int(digstart, &p, &uj, tvs != NULL) == FAIL)
 			    goto error;
 
 			arg_idx = uj;
@@ -3169,8 +3196,13 @@ vim_vsnprintf_typval(
 
 		    if (j > MAX_ALLOWED_STRING_WIDTH)
 		    {
-			format_overflow_error(digstart);
-			goto error;
+			if (tvs != NULL)
+			{
+			    format_overflow_error(digstart);
+			    goto error;
+			}
+			else
+			    j = MAX_ALLOWED_STRING_WIDTH;
 		    }
 
 		    if (j >= 0)
